@@ -155,8 +155,8 @@ class FlinkKafkaConsumer(FlinkKafkaConsumerBase):
     pull data from one or more Kafka partitions.
 
     The Flink Kafka Consumer participates in checkpointing and guarantees that no data is lost
-    during a failure, and taht the computation processes elements 'exactly once. (These guarantees
-    naturally assume that Kafka itself does not loose any data.)
+    during a failure, and that the computation processes elements 'exactly once. (These guarantees
+    naturally assume that Kafka itself does not lose any data.)
 
     Please note that Flink snapshots the offsets internally as part of its distributed checkpoints.
     The offsets committed to Kafka / Zookeeper are only to bring the outside view of progress in
@@ -365,7 +365,7 @@ class JdbcSink(SinkFunction):
                              .typeInformationToSqlType(field_type.get_java_type_info()))
         j_sql_type = to_jarray(gateway.jvm.int, sql_types)
         output_format_clz = gateway.jvm.Class\
-            .forName('org.apache.flink.connector.jdbc.internal.JdbcBatchingOutputFormat', False,
+            .forName('org.apache.flink.connector.jdbc.internal.JdbcOutputFormat', False,
                      get_gateway().jvm.Thread.currentThread().getContextClassLoader())
         j_int_array_type = to_jarray(gateway.jvm.int, []).getClass()
         j_builder_method = output_format_clz.getDeclaredMethod('createRowJdbcStatementBuilder',
